@@ -12,7 +12,7 @@ use strict;
 use warnings 'all';
 use vars qw($VERSION);
 
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 use Carp qw(croak);
 use Date::Parse qw(str2time);
@@ -29,8 +29,8 @@ sub new
   my $self = { time    => str2time($row->[0].' 00:00:00 +0000 (GMT)'),
 	       date    => _trim($row->[0]),
 	       details => _trim($row->[1]),
-	       debit   => _trim($row->[2]),
-	       credit  => _trim($row->[3]) };
+	       debit   => _trim($row->[2]),    ## no critic (ProhibitMagicNumbers)
+	       credit  => _trim($row->[3]) };  ## no critic (ProhibitMagicNumbers)
   # Balance may be undef (avoid 'Odd number of elements in anonymous hash')
   $self->{balance} = _trim($row->[4]);
   bless $self, $class;
